package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.gson.reflect.TypeToken;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.common.core.util.R;
import com.syc.slm.common.core.util.RestTemplateUtils;
import com.syc.slm.slmbi.config.DataCenterConfig;
import com.syc.slm.slmbi.dao.InstrumentPanelMapper;
import com.syc.slm.slmbi.dto.*;
import com.syc.slm.slmbi.entity.*;
import com.syc.slm.slmbi.enums.AccessType;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.net.URLDecoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

/**
 * slm-bi
 * 仪表板
 *
 * @author : lin
 * @date : 2021-03-25 16:17
 **/
@Slf4j
@Service
public class InstrumentPanelInitServiceImpl extends ServiceImpl<InstrumentPanelMapper, InstrumentPanel>
    implements InstrumentPanelInitService {

    @Value("${initGroupName:默认模板}")
    private String initGroupName;

    @Value("${initPanelName1:业务情况综合看板}")
    private String initPanelName1;

    @Value("${initPanelName2:售后服务综合看板}")
    private String initPanelName2;

    @Value("${initConfigId1:e5138f8a54415a3bac86e2be638e09c8}")
    private String initConfig1 ;

    @Value("${initConfigId2:1fffa826bc6ed38031288e2ef7a2173d}")
    private String initConfig2;

    @Autowired
    private InstrumentPanelConfigService configService;

    @Autowired
    private InstrumentPanelGroupService panelGroupService;

    @Autowired
    private InstrumentPanelService panelService;

    @Autowired
    private InstrumentPanelTreeService treeService;

    @Autowired
    private DataCenterConfig dataCenterConfig;

    @Autowired
    private InstrumentPanelGroupService groupService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String initBatchPanel(String token, HttpServletRequest request, String appId) {
        LambdaQueryWrapper<InstrumentPanelGroup> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(InstrumentPanelGroup::getName, "全部分组");
        List<InstrumentPanelGroup> list = groupService.list(where);
        list.forEach(x-> this.initOnePanel(x.getAppId(),token,request,appId));
        return "true";
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String initPanelByTenant(String token, HttpServletRequest request, String appId) {
        log.info("盛云平台创建租户初始化分组");
        log.info("appId:"+appId);
        log.info("盛云平台创建租户初始化分组");
        LambdaQueryWrapper<InstrumentPanelGroup> where = new LambdaQueryWrapper<>();
        where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.eq(BaseEntity::getAppId, appId);
        where.eq(InstrumentPanelGroup::getName, "全部分组");
        InstrumentPanelGroup one = groupService.getOne(where);
        if(ObjectUtils.isEmpty(one)) {
            InstrumentPanelGroupDTO groupDTO = new InstrumentPanelGroupDTO();
            groupDTO.setParentId(null);
            groupDTO.setName("全部分组");
            groupService.saveGroup(groupDTO, appId);
        }
        // 因为是初始化当前租户，所以queryAppId和appId是一样的
        this.initOnePanel(appId,token,request,appId);
        return "true";
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String initOnePanel(String queryAppId, String token,HttpServletRequest request,String appId) {
        // 初始化request域
        this.setRequestAttribute(token,queryAppId,appId,request);
        LambdaQueryWrapper<InstrumentPanelGroup> groupWhere = new LambdaQueryWrapper<>();
        groupWhere.eq(InstrumentPanelGroup::getName, initGroupName);
        groupWhere.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        groupWhere.eq(BaseEntity::getAppId, queryAppId);
        InstrumentPanelGroup one = panelGroupService.getOne(groupWhere);
        if(ObjectUtils.isEmpty(one)){
            LambdaQueryWrapper<InstrumentPanelGroup> where = new LambdaQueryWrapper<>();
            where.eq(InstrumentPanelGroup::getName, "全部分组");
            where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
            where.eq(BaseEntity::getAppId, queryAppId);
            InstrumentPanelGroup allGroup = panelGroupService.getOne(where);
            InstrumentPanelGroupDTO cGroup = new InstrumentPanelGroupDTO();
            cGroup.setParentId(allGroup.getId());
            cGroup.setName("默认模板");
            String groupId = panelGroupService.saveGroup(cGroup, queryAppId);
            one= panelGroupService.getById(groupId);
        }

        init(queryAppId, one, initConfig1, initPanelName1);
        init(queryAppId, one, initConfig2, initPanelName2);
        return null;
    }

    private void init(String queryAppId, InstrumentPanelGroup one, String initConfig, String initPanelName) {
        InstrumentPanelConfig panelConfig = configService.getById(initConfig);
        String config;
        try {
            config = URLDecoder.decode(panelConfig.getConfigDetails(), StandardCharsets.UTF_8.name());
        } catch (Exception e) {
            e.printStackTrace();
            throw new SysException("仪表板配置详情解密异常");
        }
        InstrumentPanelTree tree = treeService.getById(panelConfig.getPanelId());
        LambdaQueryWrapper<InstrumentPanelTree> where = new LambdaQueryWrapper<>();
        //where.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        where.likeRight(InstrumentPanelTree::getFullId, tree.getFullId() + ".");
        where.orderByAsc(InstrumentPanelTree::getAssemblyKey);
        List<InstrumentPanelTree> panelTrees = treeService.list(where);
        configService.publish(queryAppId, setConfig(one, queryAppId, initPanelName, config, panelTrees));
    }

    private InstrumentPanelConfigDTO setConfig(InstrumentPanelGroup one , String queryAppId, String panelName, String panelConfig, List<InstrumentPanelTree> panelTrees){
        LambdaQueryWrapper<InstrumentPanel> where1 = new LambdaQueryWrapper<>();
        where1.eq(BaseEntity::getAppId, queryAppId);
        where1.eq(InstrumentPanel::getName, panelName);
        where1.eq(BaseEntity::getRecordStatus, SlmConstants.DATA_VALID);
        InstrumentPanel instrumentPanel = baseMapper.selectOne(where1);
        if(ObjectUtils.isEmpty(instrumentPanel)) {
            SaveInstrumentPanelDTO panelDto = new SaveInstrumentPanelDTO();
            panelDto.setName(panelName);
            panelDto.setGroupId(one.getId());
            String panelId = panelService.savePanel(queryAppId, panelDto);
            instrumentPanel=baseMapper.selectById(panelId);
        }

        InstrumentPanelConfigDTO config = new InstrumentPanelConfigDTO();
        List<InstrumentPanelAssemblyDTO> assemblyList = Lists.newArrayList();
        panelTrees.forEach(x->{
            InstrumentPanelAssemblyDTO dto = new InstrumentPanelAssemblyDTO();
            dto.setName(x.getName());
            dto.setKey(x.getAssemblyKey());
            assemblyList.add(dto);
        });
        config.setAssemblys(assemblyList);
        config.setConfigDetails(panelConfig);
        config.setPanelId(instrumentPanel.getId());
        InstrumentPanelPermissionDTO permission = new InstrumentPanelPermissionDTO();
        permission.setAccessType(AccessType.PUBLIC.value);
        permission.setDescription("系统初始化");
        permission.setGroupId(one.getId());
        permission.setPanelId(instrumentPanel.getId());
        permission.setPanelName(instrumentPanel.getName());
        config.setPermission(permission);
        return config;
    }

    private void setRequestAttribute(String token,String queryAppId,String appId,HttpServletRequest request){
        Map<String,String> uriVariables = Maps.newHashMap();
        uriVariables.put("queryAppId",queryAppId);
        uriVariables.put("appId",appId);
        String uri = dataCenterConfig.getDcClientHost() + dataCenterConfig.getSystemAdminByAppId();
        R<CurrentUser> result = RestTemplateUtils.get(uri, new TypeToken<R<CurrentUser>>() {}.getType(), uriVariables);
        request.setAttribute(token,SlmConstants.gson.toJson(result.detach()));
    }
}
